package com.example.application.views.abstractnavigation;

import com.example.application.data.entity.Matchday;
import com.example.application.data.entity.Season;
import com.example.application.data.service.MatchdayService;
import com.example.application.data.service.SeasonService;
import com.vaadin.flow.component.*;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.select.Select;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class SeasonAndMatchdayNavigation {
    // TODO: show dropdown menus also for invalid URLs (with content that fits the situation)

    private final SeasonService seasonService;
    private final MatchdayService matchdayService;

    private final Label invalidUrlLabel = new Label();

    private final HorizontalLayout selectionLayout = new HorizontalLayout();

    private final List<Season> seasonList = new ArrayList<>();
    private final Select<Season> seasonSelect = new Select<>();

    private final List<Matchday> matchdayList = new ArrayList<>();
    private final Select<Matchday> matchdaySelect = new Select<>();
    private final String route;
    private final boolean onlyMatchdaysWithActivity;
    private Runnable runnableToBeRunAfterSelection;
    private final Button prevButton = new Button(new Icon(VaadinIcon.ARROW_LEFT));
    private final Button nextButton = new Button(new Icon(VaadinIcon.ARROW_RIGHT));

    private String seasonParam;
    private String matchdayParam;

    public SeasonAndMatchdayNavigation(String route,
                                          @Autowired SeasonService seasonService,
                                          @Autowired MatchdayService matchdayService) {
        this(route, seasonService, matchdayService, false);
    }

    public SeasonAndMatchdayNavigation(String route,
                                          @Autowired SeasonService seasonService,
                                          @Autowired MatchdayService matchdayService,
                                          boolean onlyMatchdaysWithActivity) {
        this.route = route;
        this.seasonService = seasonService;
        this.matchdayService = matchdayService;
        this.onlyMatchdaysWithActivity = onlyMatchdaysWithActivity;

        configureSelectionLayout();
    }

    private void configureSelectionLayout() {
        selectionLayout.setWidthFull();
        selectionLayout.setAlignItems(FlexComponent.Alignment.CENTER);
        selectionLayout.setJustifyContentMode(FlexComponent.JustifyContentMode.END);
        selectionLayout.add(new Label("Season:"), seasonSelect, new Label("Matchday:"), matchdaySelect);

        seasonSelect.addValueChangeListener(seasonSelectValueChangeListener());
        matchdaySelect.addValueChangeListener(matchdaySelectValueChangeListener());

        // provide data
        fillSeasonSelectWithData();
    }

    private HasValue.ValueChangeListener<? super AbstractField.ComponentValueChangeEvent<Select<Matchday>, Matchday>> matchdaySelectValueChangeListener() {
        return matchdayChangeEvent -> {
            Season season = seasonSelect.getValue();
            Matchday matchday = matchdayChangeEvent.getValue();
            if (season != null && matchday != null) {
                String seasonParam = season.toString();
                String matchdayParam = matchday.toString();
                navigate(seasonParam, matchdayParam);
            }
        };
    }

    private HasValue.ValueChangeListener<? super AbstractField.ComponentValueChangeEvent<Select<Season>, Season>> seasonSelectValueChangeListener() {
        return seasonChangeEvent -> {
            Season newSeason = seasonChangeEvent.getValue();
            if (newSeason != null) {
                String seasonParam = newSeason.toString();
                String matchdayParam = null;

                Matchday matchdayInNewSeason = null;
                Matchday matchdayInOldSeason = matchdaySelect.getValue();
                if (matchdayInOldSeason != null) {
                    matchdayParam = matchdayInOldSeason.toString();
                    matchdayInNewSeason = getMatchdayFromParam(matchdayParam, newSeason);
                }
                matchdayParam = matchdayInNewSeason == null ? "1" : matchdayParam;

                navigate(seasonParam, matchdayParam);
            }
        };
    }

    private void fillSeasonSelectWithData() {
        seasonList.clear();
        seasonList.addAll(seasonService.getAllSeasonsSorted());
        seasonSelect.setItems(seasonList);
    }

    private void fillMatchdaySelectWithData() {
        matchdayList.clear();
        List<Matchday> matchdaysToAdd = onlyMatchdaysWithActivity ?
                matchdayService.getMatchdaysWithActivityForSeasonSorted(seasonSelect.getValue()) : matchdayService.getMatchdaysForSeasonSorted(seasonSelect.getValue());
        matchdayList.addAll(matchdaysToAdd);
        matchdaySelect.setItems(matchdayList);
    }

    private boolean isMatchDayParamValid(@NonNull String matchdayParam) {
        return matchdayList.stream().anyMatch(matchday -> matchdayParam.equals(matchday.toString()));
    }

    private void navigate(String seasonParam, String matchdayParam) {
        UI.getCurrent().navigate(String.format("%s/%s/%s/", route, seasonParam, matchdayParam));
//        setParameter(seasonParam, matchdayParam);
    }

    public void setParameter(String seasonParam, String matchdayParam) {

        // TODO: handle corner case where first matchday has no active games yet: still show first matchday
        if (seasonParam == null || seasonParam.equals("") || matchdayParam == null || matchdayParam.equals("")) {
            Optional<Season> latestSeason = seasonService.getLatestSeason();
            if (latestSeason.isPresent()) {
                Optional<Matchday> latestMatchday = matchdayService.getLastMatchdayWithActivityForSeason(latestSeason.get());
                if (latestMatchday.isPresent()) {
                    seasonParam = latestSeason.get().toString();
                    matchdayParam = latestMatchday.get().toString();
                    navigate(seasonParam, matchdayParam);
                    setParameter(seasonParam, matchdayParam);
                }
            }
            return;
        }

        Season season = getSeasonFromParam(seasonParam);
        if (season == null) {
            invalidUrlLabel.setText(String.format("Invalid URL! Season \"%s\" does not exist in the database!", seasonParam));
            return;
        }

        seasonSelect.setValue(season);
        this.seasonParam = seasonParam;
        fillMatchdaySelectWithData();

        Matchday matchday = getMatchdayFromParam(matchdayParam);
        if (matchday == null) {
            String messageExtra = onlyMatchdaysWithActivity ? " or has no games played yet" : "";
            invalidUrlLabel.setText(String.format("Invalid URL! Matchday \"%s\" in Season \"%s\" does not exist in the database%s!", matchdayParam, seasonParam, messageExtra));
            return;
        }

        matchdaySelect.setValue(matchday);
        this.matchdayParam = matchdayParam;
        configureButtons();

        if ((runnableToBeRunAfterSelection) != null) {
            runnableToBeRunAfterSelection.run();
        }
    }

    @Nullable
    private Season getSeasonFromParam(@NonNull String seasonParam) {
        for (Season season : seasonList) {
            if (seasonParam.equals(season.toString())) {
                return season;
            }
        }
        return null;
    }

    @Nullable
    private Matchday getMatchdayFromParam(@NonNull String matchdayParam) {
        return getMatchdayFromParam(matchdayParam, null);
    }

    @Nullable
    private Matchday getMatchdayFromParam(@NonNull String matchdayParam, @Nullable Season season) {
        List<Matchday> matchdayList = season == null ? this.matchdayList : matchdayService.getMatchdaysForSeasonSorted(season);
        for (Matchday matchday : matchdayList) {
            if (matchdayParam.equals(matchday.toString())) {
                return matchday;
            }
        }
        return null;
    }

    public void setRunnableToBeRunAfterSelection(Runnable runnableToBeRunAfterSelection) {
        this.runnableToBeRunAfterSelection = runnableToBeRunAfterSelection;
    }


    private void configureButtons() {
        prevButton.setEnabled(isMatchDayParamValid(getPrevMatchdayParam()));
        prevButton.addClickListener(getButtonClickListener(getPrevMatchdayParam()));

        nextButton.setEnabled(isMatchDayParamValid(getNextMatchdayParam()));
        nextButton.addClickListener(getButtonClickListener(getNextMatchdayParam()));
    }

    private ComponentEventListener<ClickEvent<Button>> getButtonClickListener(String matchdayParam) {
        return buttonClickEvent -> navigate(seasonParam, matchdayParam);
    }

    private String getPrevMatchdayParam() {
        try {
            return String.valueOf(Integer.parseInt(matchdayParam) - 1);
        } catch (NumberFormatException e) {
            return "";
        }
    }

    private String getNextMatchdayParam() {
        try {
            return String.valueOf(Integer.parseInt(matchdayParam) + 1);
        } catch (NumberFormatException e) {
            return "";
        }
    }

    public Optional<Matchday> getSelectedMatchday() {
        return matchdaySelect.getOptionalValue();
    }

    public Optional<Season> getSelectedSeason() {
        return seasonSelect.getOptionalValue();
    }

    public HorizontalLayout getSelectionLayout() {
        return selectionLayout;
    }

    public Button getPrevButton() {
        return prevButton;
    }

    public Button getNextButton() {
        return nextButton;
    }

    public Label getInvalidUrlLabel() {
        return invalidUrlLabel;
    }
}
